
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_document_repository/core/identifiable.dart';
import 'package:rxdart/rxdart.dart';

abstract class DocumentRepository<Value extends Identifiable> {
  CollectionReference collection;
  BehaviorSubject<List<Value>> itemsController =
  BehaviorSubject<List<Value>>.seeded(<Value>[]);
  Stream<List<Value>> get items => itemsController.stream;

  DocumentRepository(this.collection) {
    initItems();
  }

  Value fromMap(Map data);
  Map toMap(Value value);

  List<Value> snapshotToValues(QuerySnapshot snapshot) {
    return snapshot.documents.map<Value>((doc) => fromMap(doc.data)).toList();
  }

  void initItems() async {
    var valueList = await list();
    itemsController.add(valueList);
    itemsController
        .addStream(collection.snapshots().map<List<Value>>(snapshotToValues));
  }

  Future<Value> get(dynamic id) async{
    var data = (await collection.document(id).get()).data;
    if(data == null){
      return null;
    }
    else{
      return fromMap(data);
    }
  }

  Future<Value> add(Value t) async {
    var doc = collection.document();
    t.id = doc.documentID;
    Map<String, dynamic> json = toMap(t);
    await doc.setData(json);
    return t;
  }


  Future<List<Value>> list() async {
    var documents = await listDocuments();
    var data = documents.map<Value>((d) => fromMap(d.data)).toList();
    return data;
  }

  Future<List<DocumentSnapshot>> listDocuments() async {
    QuerySnapshot result = await collection.getDocuments();
    return result.documents;
  }


  Future<bool> remove(Value t) async {
    await collection.document(t.id).delete();
  }
}
