import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_document_repository/core/document_repository.dart';
import 'package:firebase_document_repository/core/identifiable.dart';

abstract class ExternalRepository<Value extends Identifiable>
    extends DocumentRepository<Value> {
  CollectionReference internalCollection;
  ExternalRepository(CollectionReference collection, this.internalCollection)
      : super(collection);


  Future<Value> add(Value t) async {
    try {
      var model = await super.add(t);
      await internalCollection
          .document(model.id)
          .setData({"externalId": model.id});
      return model;
    } catch (e, s) {
      return null;
    }
  }


  Future<List<DocumentSnapshot>> listDocuments() async {
    var internalDocs = await super.listDocuments();
    var externalDocs = <DocumentSnapshot>[];
    for (var doc in internalDocs) {
      externalDocs.add(await collection.document(doc.documentID).get());
    }
    return externalDocs;
  }


  Future<bool> remove(Value t) async {
    try {
      await super.remove(t);
      await internalCollection.document(t.id).delete();
      return true;
    } catch (e, s) {
      return false;
    }
  }
}
