
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_document_repository/core/document_repository.dart';
import 'package:firebase_document_repository/core/identifiable.dart';

abstract class ReferenceRepository<Value extends Identifiable>
    extends DocumentRepository<Value> {
  CollectionReference _foreignCollection;
  ReferenceRepository(CollectionReference collection, this._foreignCollection)
      : super(collection);


  Future<Value> add(Value t) async {
    var doc = collection.document();
    await doc.setData(toMap(t));
    return t;
  }


  Future<List<Value>> list() async {
    List<DocumentSnapshot> docs = (await listDocuments());
    List<Future> futures = <Future>[];
    List<Value> results = <Value>[];

    for (DocumentSnapshot snap in docs) {
      futures.add(
          _foreignCollection.document(snap.data["foreignId"]).get().then((m) {
            results.add(fromMap(m.data));
          }));
    }

    await Future.wait(futures);
    return results;
  }


  Future<bool> remove(Value t) async {
    await collection.document(t.id).delete();
  }


  Map toMap(Value t) {
    Map<String, dynamic> m = Map();
    m["foreignId"] = t.id;
    return m;
  }
}
